/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <Xm/XmAll.h>

static Widget toggle, text;

static void GetValuesCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    Boolean  state;
    char    *string;

    /* Retrieve resource values by varargs function */
    XtVaGetValues(toggle, /* VVV pass the address */
                  XmNset, &state,    /* Check toggle state */
                  NULL); /* Pass NULL at the end of the arg */
    if (state) printf("Toggle is now ON.\n");
    else       printf("Toggle is now OFF.\n");

    XtVaGetValues(text, /*  VVVVVV pass the address */
                  XmNvalue, &string,    /* Get the entered text */
                  NULL); /* Pass NULL at the end of the arg */
    printf("Entered text: %s\n", string);
    XtFree(string); /* Free if not necessary */
}

main(argc, argv)
    int  argc;
    char **argv;
{
    XtAppContext app_context;
    Widget toplevel, rowcol, button;
    static char *value;
    /* You should normally set color, font and label in resource file */
    static char *color1 = "red", *font1 = "*-16-*:", *label1 = "Toggle";
    static char *color2 = "blue", *font2 = "*-24-*:", *label2 = "Button";

    XtSetLanguageProc(NULL, NULL, NULL);

    /* Set resources by varargs funcs */
    toplevel = XtVaAppInitialize(&app_context, "Varesource", NULL, 0,
                 &argc, argv, NULL,
                 XmNmaxWidth,  400,       /* Maximum allowable width */
                 XmNmaxHeight, 200,       /* Maximum allowable height */
                 NULL); /* Pass NULL at the end of the args */

    rowcol = XtVaCreateWidget("rowcol",
               xmRowColumnWidgetClass, toplevel,
               XmNmarginWidth,  20,       /* Margin width */
               XmNmarginHeight, 10,       /* Margin height */
               NULL); /* Pass NULL at the end of the args */
    XtManageChild(rowcol);

    toggle = XtVaCreateManagedWidget("toggle", 
           xmToggleButtonWidgetClass, rowcol,
           XmNset, True,              /* Set the toggle ON */
           /* Convert the resouce type */
           XtVaTypedArg, XmNbackground,  XmRString, color1, strlen(color1)+1,
           XtVaTypedArg, XmNfontList,    XmRString, font1,  strlen(font1)+1,
           XtVaTypedArg, XmNlabelString, XmRString, label1, strlen(label1)+1,
           /* Caution: You cannot free the created compound string ! */
           NULL); /* Pass NULL at the end of the args */

    value = XtMalloc(1024);    /* Allocate the space */
    strcpy(value, "Allocated Text"); /* Copy the text */

    text = XtVaCreateManagedWidget("text", 
               xmTextFieldWidgetClass, rowcol,
               XmNvalue, value,           /* Set the text */
               NULL); /* Pass NULL at the end of the args */

    XtFree(value); /* This value is no more necessary */

    button = XmCreatePushButton(rowcol, "button", NULL, 0);
    XtAddCallback(button, XmNactivateCallback, GetValuesCB, NULL);
    XtManageChild(button);
    /* Set resources by varargs funcs after creating widget */
    XtVaSetValues(button,
           XmNmarginWidth,  10,    /* left and right margin */
           XmNmarginHeight, 10,    /* top and bottom margin */
           /* Convert the resource types */
           XtVaTypedArg, XmNbackground,  XmRString, color2, strlen(color2)+1,
           XtVaTypedArg, XmNfontList,    XmRString, font2,  strlen(font2)+1,
           XtVaTypedArg, XmNlabelString, XmRString, label2, strlen(label2)+1,
           /* Caution: You cannot free the created compound string ! */
           NULL); /* Pass NULL at the end of the args */

    XtRealizeWidget(toplevel);
    XtAppMainLoop(app_context);
}
