/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <Xm/XmAll.h>
#define SCALE   5    /* scaling */

/* Called when pointer is moved on root window */
static void Radar(w, gc, event)
    Widget w;
    GC     gc;  /* Graphics Context (necessary for drawing by Xlib) */
    XEvent event;  /* Event struct of pointer motion */
{
    XClearWindow(XtDisplay(w), XtWindow(w)); /* Once clear the window */
    /* Draw a circle at the location of 1/SCALE */
    XFillArc(XtDisplay(w), XtWindow(w), gc,
             event.xmotion.x / SCALE - 5, event.xmotion.y / SCALE - 5,
             11, 11, 0, 360*64);
}

main(argc, argv)
    int  argc;
    char **argv;
{
    XtAppContext app_context;
    Widget toplevel, canvas;
    Arg    al[20];
    int    ac, screen_w, screen_h;
    XEvent event;
    GC     gc;

    XtSetLanguageProc(NULL, NULL, NULL);
    ac = 0;
    toplevel = XtAppInitialize(&app_context, "Customloop", NULL, 0,
                               &argc, argv, NULL, al, ac);
    /* Get the screen size */
    screen_w = WidthOfScreen(XtScreen(toplevel));
    screen_h = HeightOfScreen(XtScreen(toplevel));

    ac = 0;  /* The size of drawing area should be 1/SCALE of the screen */
    XtSetArg(al[ac], XmNwidth,  screen_w / SCALE); ac++;
    XtSetArg(al[ac], XmNheight, screen_h / SCALE); ac++;
    canvas = XmCreateDrawingArea(toplevel, "canvas", al, ac);
    XtManageChild(canvas);

    XtRealizeWidget(toplevel);

    /* Require the pointer motion event on root window */
    XSelectInput(XtDisplay(toplevel), RootWindowOfScreen(XtScreen(toplevel)),
                 PointerMotionMask);
    gc = XCreateGC(XtDisplay(canvas), XtWindow(canvas), NULL, NULL);
    XSetForeground(XtDisplay(canvas), gc, BlackPixelOfScreen(XtScreen(canvas)));

    for (;;) {  /* Custom event loop */
        XtAppNextEvent(app_context, &event); /* retrieve an event */
        /* If it is motion event on root window, call Radar() */
        if ( event.type == MotionNotify &&
             event.xany.window == RootWindowOfScreen(XtScreen(toplevel)) )
            Radar(canvas, gc, event);
        else  /* otherwise, process the event as usual */
            XtDispatchEvent(&event);
    }
}
